using System;
using System.IO;
using System.Collections;
using System.Text.RegularExpressions;

using NI.Email.Mime;
using NI.Email.Mime.Message;
using NI.Email.Mime.Field;

public partial class ViewAttachment : System.Web.UI.Page
{
    private MimeMessage _Message;
    
    protected MimeMessage Message
    {
        get { return _Message; }
    }
    
    protected override void OnLoad(EventArgs e)
    {
        base.OnLoad(e);
        string filename = Convert.ToString(Request["filename"]);
        if (filename.Trim() != String.Empty)
        {
            Page.MapPath(Page.AppRelativeVirtualPath);
            filename = Regex.Replace(Page.MapPath(Page.AppRelativeVirtualPath), @"\\[^.]+[.]aspx$", @"\" + filename, RegexOptions.Compiled | RegexOptions.Singleline);
            _Message = new MimeMessage(new FileStream(filename, FileMode.Open, FileAccess.Read));
        }
        bool isEmbedded = Convert.ToBoolean(Request["embedded"]);
        BodyPart bp = null;
        if (!isEmbedded)
        {
            bp = GetAttachment(Message, Convert.ToInt32(Request["attachnum"]), 0);
        }
        else
        {
            bp = GetEmbeddedAttachment(Message, Convert.ToString(Request["attachcid"]));
        }
        if (bp.Body is IBinaryBody)
        {
            Response.ContentType = bp.MimeType;

            BinaryWriter bw = new BinaryWriter(Response.OutputStream);
            Stream strm = (bp.Body as IBinaryBody).Reader.BaseStream;
            strm.Position = 0;
            ArrayList arData = new ArrayList();
            int b;
            while ((b = strm.ReadByte()) != -1)
            {
                arData.Add((byte)b);
            }

            byte[] byteData = (byte[])arData.ToArray(typeof(byte));
            bw.Write(byteData);
            bw.Flush();
            if (!isEmbedded)
            {
                Response.AddHeader("Content-Disposition", "Attachment; filename=" + Convert.ToInt32(Request["attachnum"]));
            }
        }
        else if (bp.Body is MimeMessage)
        {
            Response.ContentType = ContentTypeField.TypeTextPlain;
            (bp.Body as MimeMessage).WriteTo(Response.OutputStream);
            if (!isEmbedded)
            {
                Response.AddHeader("Content-Disposition", "Attachment; filename=" + Convert.ToInt32(Request["attachnum"]));
            }
        }
        else if (bp.Body is Multipart)
        {
            Response.ContentType = ContentTypeField.TypeTextPlain;
            (bp.Body as Multipart).WriteTo(Response.OutputStream);
            if (!isEmbedded)
            {
                Response.AddHeader("Content-Disposition", "Attachment; filename=" + Convert.ToInt32(Request["attachnum"]));
            }
        }
        else
        {
            Response.ContentType = ContentTypeField.TypeTextHtml;
            StreamWriter sw = new StreamWriter(Response.OutputStream);
            sw.Write("<html><head/><body/></html>");
            sw.Flush();
        }
        Response.OutputStream.Flush();
        Response.End();
    }

    protected BodyPart GetAttachment(Entity mimeEntity, int num, int indexOffset)
    {
        if (mimeEntity.IsMultipart)
        {
            IList bps = ((Multipart)mimeEntity.Body).BodyParts;
            for (int i = 0; i < bps.Count; ++i)
            {
                BodyPart currentBp = bps[i] as BodyPart;
                ContentTypeField contentType = currentBp.Header.GetField(MimeField.ContentType) as ContentTypeField;
                if (contentType == null) continue;

                if (currentBp.Body is MimeMessage)
                {
                    if (num == i + indexOffset)
                    {
                        return currentBp;
                    }
                    else
                    {
                        return GetAttachment((MimeMessage)currentBp.Body, num, i + indexOffset);
                    }
                }
                else if (!(currentBp.Body is ITextBody))
                {
                    if (num == i + indexOffset)
                    {
                        return currentBp;
                    }
                }
            }
        }
        return null;
    }

    protected BodyPart GetEmbeddedAttachment(Entity mimeEntity, string cid)
    {
        if (mimeEntity.IsMultipart)
        {
            IList bps = ((Multipart)mimeEntity.Body).BodyParts;
            foreach (Entity part in ((Multipart)mimeEntity.Body).BodyParts)
            {
                ContentTypeField contentType = part.Header.GetField(MimeField.ContentType) as ContentTypeField;
                if (contentType == null) continue;
                MimeField cidField = part.Header.GetField("Content-Id");
                string currentCid = cidField != null ? cidField.Body.Trim() : String.Empty;
                if (part.Body is IBinaryBody && cid == currentCid)
                {
                    return part as BodyPart;
                }
                else if (part.Body is MimeMessage)
                {
                    return GetEmbeddedAttachment((MimeMessage)part.Body, cid);
                }
            } 
        }
        return null;
    }
}
